from __future__ import annotations
import ast
from typing import Tuple, TypeVar, Iterable
from func_adl import ObjectStream, func_adl_callback, func_adl_parameterized_call
from enum import Enum
import func_adl_servicex_xaodr22

_method_map = {
    'getNumberOfSamplings': {
        'metadata_type': 'add_method_type_info',
        'type_string': 'CaloSampling',
        'method_name': 'getNumberOfSamplings',
        'return_type': 'unsigned int',
    },
    'getSamplingPattern': {
        'metadata_type': 'add_method_type_info',
        'type_string': 'CaloSampling',
        'method_name': 'getSamplingPattern',
        'return_type': 'unsigned int',
    },
    'barrelPattern': {
        'metadata_type': 'add_method_type_info',
        'type_string': 'CaloSampling',
        'method_name': 'barrelPattern',
        'return_type': 'unsigned int',
    },
    'endcapPattern': {
        'metadata_type': 'add_method_type_info',
        'type_string': 'CaloSampling',
        'method_name': 'endcapPattern',
        'return_type': 'unsigned int',
    },
    'getSamplingName': {
        'metadata_type': 'add_method_type_info',
        'type_string': 'CaloSampling',
        'method_name': 'getSamplingName',
        'return_type_element': '__gnu_cxx::__normal_iterator<char*,string>',
        'return_type_collection': 'string',
    },
    'getSampling': {
        'metadata_type': 'add_method_type_info',
        'type_string': 'CaloSampling',
        'method_name': 'getSampling',
        'return_type': 'CaloSampling::CaloSample',
    },
}

_enum_map = {
    'getSamplingPattern': [
        {
            'metadata_type': 'define_enum',
            'namespace': 'CaloSampling',
            'name': 'CaloSample',
            'values': [
                'PreSamplerB',
                'EMB1',
                'EMB2',
                'EMB3',
                'PreSamplerE',
                'EME1',
                'EME2',
                'EME3',
                'HEC0',
                'HEC1',
                'HEC2',
                'HEC3',
                'TileBar0',
                'TileBar1',
                'TileBar2',
                'TileGap1',
                'TileGap2',
                'TileGap3',
                'TileExt0',
                'TileExt1',
                'TileExt2',
                'FCAL0',
                'FCAL1',
                'FCAL2',
                'MINIFCAL0',
                'MINIFCAL1',
                'MINIFCAL2',
                'MINIFCAL3',
                'Unknown',
            ],
        },
    ],
    'getSamplingName': [
        {
            'metadata_type': 'define_enum',
            'namespace': 'CaloSampling',
            'name': 'CaloSample',
            'values': [
                'PreSamplerB',
                'EMB1',
                'EMB2',
                'EMB3',
                'PreSamplerE',
                'EME1',
                'EME2',
                'EME3',
                'HEC0',
                'HEC1',
                'HEC2',
                'HEC3',
                'TileBar0',
                'TileBar1',
                'TileBar2',
                'TileGap1',
                'TileGap2',
                'TileGap3',
                'TileExt0',
                'TileExt1',
                'TileExt2',
                'FCAL0',
                'FCAL1',
                'FCAL2',
                'MINIFCAL0',
                'MINIFCAL1',
                'MINIFCAL2',
                'MINIFCAL3',
                'Unknown',
            ],
        },
    ],      
}

T = TypeVar('T')


def _add_method_metadata(s: ObjectStream[T], a: ast.Call) -> Tuple[ObjectStream[T], ast.Call]:
    '''Add metadata for a collection to the func_adl stream if we know about it
    '''
    assert isinstance(a.func, ast.Attribute)
    if a.func.attr in _method_map:
        s_update = s.MetaData(_method_map[a.func.attr])
        s_update = s_update.MetaData({
            'metadata_type': 'inject_code',
            'name': 'CaloGeoHelpers/CaloSampling.h',
            'body_includes': ["CaloGeoHelpers/CaloSampling.h"],
        })
        for md in _enum_map.get(a.func.attr, []):
            s_update = s_update.MetaData(md)
        return s_update, a
    else:
        return s, a


@func_adl_callback(_add_method_metadata)
class CaloSampling:
    "A class"

    class CaloSample(Enum):
        PreSamplerB = 0
        EMB1 = 1
        EMB2 = 2
        EMB3 = 3
        PreSamplerE = 4
        EME1 = 5
        EME2 = 6
        EME3 = 7
        HEC0 = 8
        HEC1 = 9
        HEC2 = 10
        HEC3 = 11
        TileBar0 = 12
        TileBar1 = 13
        TileBar2 = 14
        TileGap1 = 15
        TileGap2 = 16
        TileGap3 = 17
        TileExt0 = 18
        TileExt1 = 19
        TileExt2 = 20
        FCAL0 = 21
        FCAL1 = 22
        FCAL2 = 23
        MINIFCAL0 = 24
        MINIFCAL1 = 25
        MINIFCAL2 = 26
        MINIFCAL3 = 27
        Unknown = 28


    def getNumberOfSamplings(self) -> int:
        "A method"
        ...

    def getSamplingPattern(self, s: CaloSampling.CaloSample) -> int:
        "A method"
        ...

    def barrelPattern(self) -> int:
        "A method"
        ...

    def endcapPattern(self) -> int:
        "A method"
        ...

    def getSamplingName(self, theSample: CaloSampling.CaloSample) -> func_adl_servicex_xaodr22.str.str:
        "A method"
        ...

    def getSampling(self, name: func_adl_servicex_xaodr22.str.str) -> func_adl_servicex_xaodr22.calosampling.CaloSampling.CaloSample:
        "A method"
        ...
