from __future__ import annotations
import ast
from typing import Tuple, TypeVar, Iterable
from func_adl import ObjectStream, func_adl_callback, func_adl_parameterized_call
from enum import Enum
import func_adl_servicex_xaodr22

_method_map = {
    'contentsPixelClusEndcapC': {
        'metadata_type': 'add_method_type_info',
        'type_string': 'xAOD::TrigSpacePointCounts_v1',
        'method_name': 'contentsPixelClusEndcapC',
        'return_type_element': 'float',
        'return_type_collection': 'const vector<float>',
    },
    'contentsPixelClusBarrel': {
        'metadata_type': 'add_method_type_info',
        'type_string': 'xAOD::TrigSpacePointCounts_v1',
        'method_name': 'contentsPixelClusBarrel',
        'return_type_element': 'float',
        'return_type_collection': 'const vector<float>',
    },
    'contentsPixelClusEndcapA': {
        'metadata_type': 'add_method_type_info',
        'type_string': 'xAOD::TrigSpacePointCounts_v1',
        'method_name': 'contentsPixelClusEndcapA',
        'return_type_element': 'float',
        'return_type_collection': 'const vector<float>',
    },
    'pixelClusTotBins': {
        'metadata_type': 'add_method_type_info',
        'type_string': 'xAOD::TrigSpacePointCounts_v1',
        'method_name': 'pixelClusTotBins',
        'return_type': 'unsigned int',
    },
    'pixelClusTotMin': {
        'metadata_type': 'add_method_type_info',
        'type_string': 'xAOD::TrigSpacePointCounts_v1',
        'method_name': 'pixelClusTotMin',
        'return_type': 'float',
    },
    'pixelClusTotMax': {
        'metadata_type': 'add_method_type_info',
        'type_string': 'xAOD::TrigSpacePointCounts_v1',
        'method_name': 'pixelClusTotMax',
        'return_type': 'float',
    },
    'pixelClusSizeBins': {
        'metadata_type': 'add_method_type_info',
        'type_string': 'xAOD::TrigSpacePointCounts_v1',
        'method_name': 'pixelClusSizeBins',
        'return_type': 'unsigned int',
    },
    'pixelClusSizeMin': {
        'metadata_type': 'add_method_type_info',
        'type_string': 'xAOD::TrigSpacePointCounts_v1',
        'method_name': 'pixelClusSizeMin',
        'return_type': 'float',
    },
    'pixelClusSizeMax': {
        'metadata_type': 'add_method_type_info',
        'type_string': 'xAOD::TrigSpacePointCounts_v1',
        'method_name': 'pixelClusSizeMax',
        'return_type': 'float',
    },
    'sctSpEndcapC': {
        'metadata_type': 'add_method_type_info',
        'type_string': 'xAOD::TrigSpacePointCounts_v1',
        'method_name': 'sctSpEndcapC',
        'return_type': 'unsigned int',
    },
    'sctSpBarrel': {
        'metadata_type': 'add_method_type_info',
        'type_string': 'xAOD::TrigSpacePointCounts_v1',
        'method_name': 'sctSpBarrel',
        'return_type': 'unsigned int',
    },
    'sctSpEndcapA': {
        'metadata_type': 'add_method_type_info',
        'type_string': 'xAOD::TrigSpacePointCounts_v1',
        'method_name': 'sctSpEndcapA',
        'return_type': 'unsigned int',
    },
    'pixelClusBarrelSumEntries': {
        'metadata_type': 'add_method_type_info',
        'type_string': 'xAOD::TrigSpacePointCounts_v1',
        'method_name': 'pixelClusBarrelSumEntries',
        'return_type': 'double',
    },
    'pixelClusEndcapASumEntries': {
        'metadata_type': 'add_method_type_info',
        'type_string': 'xAOD::TrigSpacePointCounts_v1',
        'method_name': 'pixelClusEndcapASumEntries',
        'return_type': 'double',
    },
    'pixelClusEndcapCSumEntries': {
        'metadata_type': 'add_method_type_info',
        'type_string': 'xAOD::TrigSpacePointCounts_v1',
        'method_name': 'pixelClusEndcapCSumEntries',
        'return_type': 'double',
    },
    'index': {
        'metadata_type': 'add_method_type_info',
        'type_string': 'xAOD::TrigSpacePointCounts_v1',
        'method_name': 'index',
        'return_type': 'int',
    },
    'usingPrivateStore': {
        'metadata_type': 'add_method_type_info',
        'type_string': 'xAOD::TrigSpacePointCounts_v1',
        'method_name': 'usingPrivateStore',
        'return_type': 'bool',
    },
    'usingStandaloneStore': {
        'metadata_type': 'add_method_type_info',
        'type_string': 'xAOD::TrigSpacePointCounts_v1',
        'method_name': 'usingStandaloneStore',
        'return_type': 'bool',
    },
    'hasStore': {
        'metadata_type': 'add_method_type_info',
        'type_string': 'xAOD::TrigSpacePointCounts_v1',
        'method_name': 'hasStore',
        'return_type': 'bool',
    },
    'hasNonConstStore': {
        'metadata_type': 'add_method_type_info',
        'type_string': 'xAOD::TrigSpacePointCounts_v1',
        'method_name': 'hasNonConstStore',
        'return_type': 'bool',
    },
    'clearDecorations': {
        'metadata_type': 'add_method_type_info',
        'type_string': 'xAOD::TrigSpacePointCounts_v1',
        'method_name': 'clearDecorations',
        'return_type': 'bool',
    },
    'trackIndices': {
        'metadata_type': 'add_method_type_info',
        'type_string': 'xAOD::TrigSpacePointCounts_v1',
        'method_name': 'trackIndices',
        'return_type': 'bool',
    },
    'auxdataConst': {
        'metadata_type': 'add_method_type_info',
        'type_string': 'xAOD::TrigSpacePointCounts_v1',
        'method_name': 'auxdataConst',
        'return_type': 'U',
    },
    'isAvailable': {
        'metadata_type': 'add_method_type_info',
        'type_string': 'xAOD::TrigSpacePointCounts_v1',
        'method_name': 'isAvailable',
        'return_type': 'bool',
    },
}

_enum_map = {      
}

T = TypeVar('T')


def _add_method_metadata(s: ObjectStream[T], a: ast.Call) -> Tuple[ObjectStream[T], ast.Call]:
    '''Add metadata for a collection to the func_adl stream if we know about it
    '''
    assert isinstance(a.func, ast.Attribute)
    if a.func.attr in _method_map:
        s_update = s.MetaData(_method_map[a.func.attr])
        s_update = s_update.MetaData({
            'metadata_type': 'inject_code',
            'name': 'xAODTrigMinBias/versions/TrigSpacePointCounts_v1.h',
            'body_includes': ["xAODTrigMinBias/versions/TrigSpacePointCounts_v1.h"],
        })
        for md in _enum_map.get(a.func.attr, []):
            s_update = s_update.MetaData(md)
        return s_update, a
    else:
        return s, a


@func_adl_callback(_add_method_metadata)
class TrigSpacePointCounts_v1:
    "A class"


    def contentsPixelClusEndcapC(self) -> func_adl_servicex_xaodr22.vector_float_.vector_float_:
        "A method"
        ...

    def contentsPixelClusBarrel(self) -> func_adl_servicex_xaodr22.vector_float_.vector_float_:
        "A method"
        ...

    def contentsPixelClusEndcapA(self) -> func_adl_servicex_xaodr22.vector_float_.vector_float_:
        "A method"
        ...

    def pixelClusTotBins(self) -> int:
        "A method"
        ...

    def pixelClusTotMin(self) -> float:
        "A method"
        ...

    def pixelClusTotMax(self) -> float:
        "A method"
        ...

    def pixelClusSizeBins(self) -> int:
        "A method"
        ...

    def pixelClusSizeMin(self) -> float:
        "A method"
        ...

    def pixelClusSizeMax(self) -> float:
        "A method"
        ...

    def sctSpEndcapC(self) -> int:
        "A method"
        ...

    def sctSpBarrel(self) -> int:
        "A method"
        ...

    def sctSpEndcapA(self) -> int:
        "A method"
        ...

    def pixelClusBarrelSumEntries(self, value_x: float, value_y: float, cutType: int) -> float:
        "A method"
        ...

    def pixelClusEndcapASumEntries(self, value_x: float, value_y: float, cutType: int) -> float:
        "A method"
        ...

    def pixelClusEndcapCSumEntries(self, value_x: float, value_y: float, cutType: int) -> float:
        "A method"
        ...

    def index(self) -> int:
        "A method"
        ...

    def usingPrivateStore(self) -> bool:
        "A method"
        ...

    def usingStandaloneStore(self) -> bool:
        "A method"
        ...

    def hasStore(self) -> bool:
        "A method"
        ...

    def hasNonConstStore(self) -> bool:
        "A method"
        ...

    def clearDecorations(self) -> bool:
        "A method"
        ...

    def trackIndices(self) -> bool:
        "A method"
        ...

    @func_adl_parameterized_call(lambda s, a, param_1: func_adl_servicex_xaodr22.type_support.cpp_generic_1arg_callback('auxdataConst', s, a, param_1))
    @property
    def auxdataConst(self) -> func_adl_servicex_xaodr22.type_support.index_type_forwarder[func_adl_servicex_xaodr22.str.str]:
        "A method"
        ...

    @func_adl_parameterized_call(lambda s, a, param_1: func_adl_servicex_xaodr22.type_support.cpp_generic_1arg_callback('isAvailable', s, a, param_1))
    @property
    def isAvailable(self) -> func_adl_servicex_xaodr22.type_support.index_type_forwarder[func_adl_servicex_xaodr22.str.str]:
        "A method"
        ...
